local FactorioUtils = require("__core__.lualib.util")
local CollisionMaskUtil = require("__core__.lualib.collision-mask-util")

local embeddedRailSignalCollisionLayer = CollisionMaskUtil.get_first_unused_layer()

-- New Rail Signal
local embeddedRailSignal = FactorioUtils.table.deepcopy(data.raw["rail-signal"]["rail-signal"])
embeddedRailSignal.name = "embedded-rail-signal"
embeddedRailSignal.collision_mask = {embeddedRailSignalCollisionLayer}
embeddedRailSignal.rail_piece = nil
embeddedRailSignal.group = "logistics"
embeddedRailSignal.subgroup = "train-transport"
embeddedRailSignal.order = "z-a[train-system]-d[rail-signal]"
embeddedRailSignal.selection_box = {{0.5, -0.5}, {1.5, 0.5}} -- shifted -1 on the x
embeddedRailSignal.animation = {
    layers = {
        {
            filename = "__base__/graphics/entity/rail-signal/rail-signal.png",
            priority = "high",
            width = 96,
            height = 96,
            frame_count = 3,
            direction_count = 8,
            shift = {-1, 0}, -- This is changed to move towards the center, in one vertical orientation.
            hr_version = {
                filename = "__base__/graphics/entity/rail-signal/hr-rail-signal.png",
                priority = "high",
                width = 192,
                height = 192,
                frame_count = 3,
                direction_count = 8,
                scale = 0.5,
                shift = {-1, 0} -- This is changed to move towards the center, in one vertical orientation.
            }
        },
        {
            filename = "__base__/graphics/entity/rail-signal/rail-signal-light.png",
            priority = "low",
            blend_mode = "additive",
            draw_as_light = true,
            width = 96,
            height = 96,
            frame_count = 3,
            direction_count = 8,
            shift = {-1, 0}, -- This is changed to move towards the center, in one vertical orientation.
            hr_version = {
                filename = "__base__/graphics/entity/rail-signal/hr-rail-signal-light.png",
                priority = "low",
                blend_mode = "additive",
                draw_as_light = true,
                width = 192,
                height = 192,
                frame_count = 3,
                direction_count = 8,
                scale = 0.5,
                shift = {-1, 0} -- This is changed to move towards the center, in one vertical orientation.
            }
        }
    }
}

-- Update all other rail signals to include our bespoke collision layer.
for _, prototypeTypeName in pairs({"rail-signal", "rail-chain-signal"}) do
    for _, prototype in pairs(data.raw[prototypeTypeName]) do
        local newMask = CollisionMaskUtil.get_mask(prototype)
        CollisionMaskUtil.add_layer(newMask, embeddedRailSignalCollisionLayer)
        prototype.collision_mask = newMask
    end
end

data:extend({embeddedRailSignal})
